@@ -93,6 +93,7 @@ class SQLiteDBTable:
     _types: List[type] = []
     _pk: str = 'id'
     _auto_increment: bool = True
+    _column_order_cache: Optional[List[str]] = None  # Cache for actual column order
 
     def __init_subclass__(cls, **kwargs):
         super().__init_subclass__(**kwargs)
@@ -104,12 +105,19 @@ class SQLiteDBTable:
     @classmethod
     def set_db(cls, db: SQLiteDB):
         cls._db = db
+        cls._column_order_cache = None  # Reset cache when database changes
         # Ensure table exists
         if not db.table_exists(cls._table):
             fields_def = ', '.join([f"{field} TEXT" for field in cls._fields])
             pk_def = f"{cls._pk} TEXT PRIMARY KEY" if not cls._auto_increment else f"{cls._pk} INTEGER PRIMARY KEY AUTOINCREMENT"
             create_table_sql = f"CREATE TABLE IF NOT EXISTS {cls._table} ({pk_def}, {fields_def})"
             db.create_table(create_table_sql)
+        # Check for missing columns and add them
+        existing_columns = [col[1] for col in db.fetchall(f"PRAGMA table_info({cls._table})")]
+        for field in cls._fields:
+            if field not in existing_columns:
+                db.execute(f"ALTER TABLE {cls._table} ADD COLUMN {field} TEXT", commit=True)
+                cls._column_order_cache = None  # Reset cache when schema changes
 
     @classmethod
     def all(cls: Type[T]) -> List[T]:
@@ -132,47 +140,80 @@ class SQLiteDBTable:
         if not row:
             return None
         obj = cls()
-        fields = [cls._pk] + cls._fields
-        for i, field in enumerate(fields):
-            if i == 0 and field == cls._pk:
-                if cls._types[i] == int:
-                    setattr(obj, field, int(row[i])
-                            if row[i] is not None else None)
-                elif cls._types[i] == str:
-                    setattr(obj, field, str(row[i])
-                            if row[i] is not None else None)
-                continue
-            if cls._types[i] == str:
-                if (row[i].startswith('[') or row[i].startswith('{')):
-                    try:
-                        setattr(obj, field, json.loads(row[i]))
-                    except json.JSONDecodeError:
-                        setattr(obj, field, row[i])
-                else:
-                    setattr(obj, field, str(row[i])
-                            if row[i] is not None else None)
-            elif cls._types[i] == list:
-                try:
-                    setattr(obj, field, json.loads(row[i]) if row[i] else [])
-                except json.JSONDecodeError:
-                    setattr(obj, field, [])
-            elif cls._types[i] == int:
-                setattr(obj, field, int(row[i])
-                        if row[i] is not None else None)
-            elif cls._types[i] == float:
-                setattr(obj, field, float(row[i])
-                        if row[i] is not None else None)
-            elif cls._types[i] == bool:
-                setattr(obj, field, bool(row[i])
-                        if row[i] is not None else None)
-            elif cls._types[i] == dict:
-                try:
-                    setattr(obj, field, json.loads(row[i]) if row[i] else {})
-                except json.JSONDecodeError:
-                    setattr(obj, field, {})
-            else:
-                setattr(obj, field, row[i])
+        
+        try:
+            # Get actual column order from database schema
+            actual_columns = cls.get_actual_column_order()
+            expected_fields = [cls._pk] + cls._fields
+            
+            # Create a mapping from actual column positions to expected field positions
+            column_mapping = {}
+            for i, actual_col in enumerate(actual_columns):
+                if actual_col in expected_fields:
+                    expected_index = expected_fields.index(actual_col)
+                    column_mapping[i] = expected_index
+            
+            # Process each column in the row based on the mapping
+            for actual_pos, row_value in enumerate(row):
+                if actual_pos not in column_mapping:
+                    continue  # Skip unknown columns
+                    
+                expected_pos = column_mapping[actual_pos]
+                field = expected_fields[expected_pos]
+                field_type = cls._types[expected_pos]
+                
+                cls._set_field_value(obj, field, field_type, row_value, expected_pos == 0 and field == cls._pk)
+                
+        except Exception as e:
+            # Fallback to original behavior if schema-based mapping fails
+            logger.warning(f"Column mapping failed for {cls._table}, falling back to positional mapping: {e}")
+            expected_fields = [cls._pk] + cls._fields
+            for i, field in enumerate(expected_fields):
+                if i >= len(row):
+                    break  # Safety check
+                field_type = cls._types[i]
+                cls._set_field_value(obj, field, field_type, row[i], i == 0 and field == cls._pk)
+                    
         return obj
+    
+    @classmethod
+    def _set_field_value(cls, obj, field: str, field_type: type, row_value, is_pk: bool = False):
+        """Helper method to set field value with proper type conversion."""
+        if is_pk:
+            if field_type is int:
+                setattr(obj, field, int(row_value) if row_value is not None else None)
+            elif field_type is str:
+                setattr(obj, field, str(row_value) if row_value is not None else None)
+            return
+            
+        if field_type is str:
+            if not row_value:
+                setattr(obj, field, "")
+            elif isinstance(row_value, str) and (row_value.startswith('[') or row_value.startswith('{')):
+                try:
+                    setattr(obj, field, json.loads(row_value))
+                except json.JSONDecodeError:
+                    setattr(obj, field, row_value)
+            else:
+                setattr(obj, field, str(row_value) if row_value is not None else None)
+        elif field_type is list:
+            try:
+                setattr(obj, field, json.loads(row_value) if row_value else [])
+            except json.JSONDecodeError:
+                setattr(obj, field, [])
+        elif field_type is int:
+            setattr(obj, field, int(row_value) if row_value is not None else None)
+        elif field_type is float:
+            setattr(obj, field, float(row_value) if row_value is not None else None)
+        elif field_type is bool:
+            setattr(obj, field, bool(row_value) if row_value is not None else None)
+        elif field_type is dict:
+            try:
+                setattr(obj, field, json.loads(row_value) if row_value else {})
+            except json.JSONDecodeError:
+                setattr(obj, field, {})
+        else:
+            setattr(obj, field, row_value)
 
     def save(self, retry=1):
         try:
@@ -212,7 +253,7 @@ class SQLiteDBTable:
     def add(self, retry=1):
         try:
             pk_val = getattr(self, self._pk, None)
-            if cls._auto_increment:
+            if self._auto_increment:
                 self.save()
             elif pk_val is None:
                 raise ValueError(
@@ -238,9 +279,9 @@ class SQLiteDBTable:
             
     def add_column(self, column_name: str, new_column_type: str = "TEXT"):
         try:
-            index = self._fields.index(column_name) + 1
             self._db.execute(
                 f"ALTER TABLE {self._table} ADD COLUMN {column_name} {new_column_type}", commit=True)
+            self.__class__._column_order_cache = None  # Reset cache when schema changes
             logger.info(f"Added column {column_name} to {self._table}")
         except sqlite3.OperationalError as e:
             if "duplicate column name" in str(e):
@@ -279,11 +320,13 @@ class SQLiteDBTable:
     def rename_column(cls, old_column: str, new_column: str):
         cls._db.execute(
             f"ALTER TABLE {cls._table} RENAME COLUMN {old_column} TO {new_column}", commit=True)
+        cls._column_order_cache = None  # Reset cache when schema changes
         
     @classmethod
     def drop_column(cls, column_name: str):
         cls._db.execute(
             f"ALTER TABLE {cls._table} DROP COLUMN {column_name}", commit=True)
+        cls._column_order_cache = None  # Reset cache when schema changes
         
     @classmethod
     def get_column_type(cls, column_name: str) -> Optional[str]:
@@ -308,6 +351,36 @@ class SQLiteDBTable:
             f"UPDATE {cls._table} SET {new_column} = CAST({old_column} AS {new_type})", commit=True)
         cls._db.execute(
             f"ALTER TABLE {cls._table} DROP COLUMN {old_column}", commit=True)
+        cls._column_order_cache = None  # Reset cache when schema changes
+        
+    @classmethod
+    def get_actual_column_order(cls) -> List[str]:
+        """Get the actual column order from the database schema."""
+        if cls._column_order_cache is not None:
+            return cls._column_order_cache
+            
+        # Use direct database access to avoid recursion through from_row()
+        with cls._db._lock:
+            import sqlite3
+            with sqlite3.connect(cls._db.db_path, check_same_thread=False) as conn:
+                cursor = conn.cursor()
+                cursor.execute(f"PRAGMA table_info({cls._table})")
+                columns_info = cursor.fetchall()
+                
+        # Each row is (cid, name, type, notnull, dflt_value, pk)
+        # Sort by column id (cid) to get the actual order
+        sorted_columns = sorted(columns_info, key=lambda x: x[0])
+        column_order = [col[1] for col in sorted_columns]
+        
+        # Cache the result
+        cls._column_order_cache = column_order
+        return column_order
+        
+    @classmethod
+    def get_expected_column_list(cls) -> str:
+        """Get comma-separated list of columns in expected order for explicit SELECT queries."""
+        expected_fields = [cls._pk] + cls._fields
+        return ', '.join(expected_fields)
 
 
 class AIModelsTable(SQLiteDBTable):
@@ -378,9 +451,9 @@ class AIModelsTable(SQLiteDBTable):
 class GameLinesTable(SQLiteDBTable):
     _table = 'game_lines'
     _fields = ['game_name', 'line_text', 'screenshot_in_anki',
-               'audio_in_anki', 'screenshot_path', 'audio_path', 'replay_path', 'translation', 'timestamp']
+               'audio_in_anki', 'screenshot_path', 'audio_path', 'replay_path', 'translation', 'timestamp', 'original_game_name']
     _types = [str,  # Includes primary key type
-              str, str, str, str, str, str, str, str, float]
+              str, str, str, str, str, str, str, str, float, str]
     _pk = 'id'
     _auto_increment = False  # Use string IDs
 
@@ -394,7 +467,8 @@ class GameLinesTable(SQLiteDBTable):
                  screenshot_path: Optional[str] = None,
                  audio_path: Optional[str] = None,
                  replay_path: Optional[str] = None,
-                 translation: Optional[str] = None):
+                 translation: Optional[str] = None,
+                 original_game_name: Optional[str] = None):
         self.id = id
         self.game_name = game_name
         self.line_text = line_text
@@ -406,6 +480,7 @@ class GameLinesTable(SQLiteDBTable):
         self.audio_path = audio_path if audio_path is not None else ''
         self.replay_path = replay_path if replay_path is not None else ''
         self.translation = translation if translation is not None else ''
+        self.original_game_name = original_game_name if original_game_name is not None else ''
 
     @classmethod
     def get_all_lines_for_scene(cls, game_name: str) -> List['GameLinesTable']:
