@@ -188,8 +188,10 @@ class OverlayProcessor:
         For primary monitor, excludes taskbar. For others, returns full monitor area.
         monitor_index: 0 = primary monitor, 1+ = others (as in mss.monitors).
         """
+        # set_dpi_awareness()
         with mss.mss() as sct:
             monitors = sct.monitors[1:]
+            # return monitors[monitor_index] if 0 <= monitor_index < len(monitors) else monitors[0]
             if is_windows() and monitor_index == 0:
                 from ctypes import wintypes
                 import ctypes
@@ -309,11 +311,9 @@ class OverlayProcessor:
                 
                 score = fuzz.ratio(text_str, self.last_oneocr_result)
                 if score >= 80:
-                    logger.info("OneOCR results are similar to the last results (score: %d). Skipping overlay update.", score)
                     return
             self.last_oneocr_result = text_str
 
-            logger.info("Sending OneOCR results to overlay.")
             await send_word_coordinates_to_overlay(self._convert_oneocr_results_to_percentages(oneocr_results, monitor_width, monitor_height))
             
             # If User Home is beangate
@@ -322,7 +322,7 @@ class OverlayProcessor:
                     f.write(json.dumps(oneocr_results, ensure_ascii=False, indent=2))
             
             if get_config().overlay.engine == OverlayEngine.ONEOCR.value and self.oneocr:
-                logger.info("Using OneOCR results for overlay as configured.")
+                logger.info("Sent %d text boxes to overlay.", len(oneocr_results))
                 return
 
             # 3. Create a composite image with only the detected text regions
@@ -371,8 +371,9 @@ class OverlayProcessor:
             crop_height=composite_image.height,
             use_percentages=True
         )
-        logger.info("Sending Google Lens results to overlay.")
         await send_word_coordinates_to_overlay(extracted_data)
+        
+        logger.info("Sent %d text boxes to overlay.", len(extracted_data))
 
     def _extract_text_with_pixel_boxes(
         self,
